import java.util.Arrays;

/**
 * Ein kleiner Test, wie String zusammengehngt werden knnen. Bei Objekten der
 * Klasse String wird jedesmal ein komplett neuer String erzeugt, was sehr
 * ineffizient werden kann. StringBuffer und StringBuilder ermglichen es,
 * einzelne Strings hinten an dranzuhngen ohne jedesmal ein neues Objekt zu
 * erstellen. Der Unterschied zwischen StringBuffer und StringBuilder ist, dass
 * die Methoden von StringBuilder nicht synchronized sind, StringBuffer aber
 * schon. D.h. StringBuffer ist Thread-safe
 */
public class StringConcatenation {

	private static int n = 10000; // Grsse des Arrays
	private static int[] array = new int[n];
	
	private static int count = 10; // Anzahl Tests
	
	public static void main(String[] args) {
		for (int i=0;i<n;i++) {
			array[i] = i;
		}

		long minString = Long.MAX_VALUE;
		long minStringBuffer = Long.MAX_VALUE;
		long minStringBuilder = Long.MAX_VALUE;
		long minArraystoString = Long.MAX_VALUE; 
		
		for (int i=0;i<count;i++) {
			minString = Math.min(minString,concatString(array));
		}
		
		for (int i=0;i<count;i++) {
			minStringBuffer = Math.min(minStringBuffer,concatStringBuffer(array));
		}
		
		for (int i=0;i<count;i++) {
			minStringBuilder = Math.min(minStringBuilder,concatStringBuilder(array));
		}
		
		for (int i=0;i<count;i++) {
			minArraystoString = Math.min(minArraystoString,concatArraystoString(array));
		}

		System.out.println("Zeit fr String: "+minString+" mms");
		System.out.println("Zeit fr StringBuffer: "+minStringBuffer+" mms");
		System.out.println("Zeit fr StringBuilder: "+minStringBuilder+" mms");
		System.out.println("Zeit fr Arrays.toString: "+minArraystoString+" mms");
	}
	
	private static long concatString(int[] array) {
		long t0 = System.nanoTime();
		String s = "";
		for (int i=0;i<array.length;i++) {
			s += array[i]+" ";
		}
		long dt = System.nanoTime()-t0;
		return dt/1000;
	}
	
	private static long concatStringBuffer(int[] array) {
		long t0 = System.nanoTime();
		StringBuffer s = new StringBuffer("");
		for (int i=0;i<array.length;i++) {
			s.append(array[i]+" ");
		}
		long dt = System.nanoTime()-t0;
		return dt/1000;
	}
	
	private static long concatStringBuilder(int[] array) {
		long t0 = System.nanoTime();
		StringBuilder s = new StringBuilder("");
		for (int i=0;i<array.length;i++) {
			s.append(array[i]+" ");
		}
		long dt = System.nanoTime()-t0;
		return dt/1000;
	}
	
	private static long concatArraystoString(int[] array) {
		long t0 = System.nanoTime();
		Arrays.toString(array);
		long dt = System.nanoTime()-t0;
		return dt/1000;
	}
}
